@@ -1,9 +1,12 @@
+pub mod approve;
 pub mod buffer_append_ix;
 pub mod buffer_close;
 pub mod buffer_execute_bundle;
 pub mod buffer_finalize;
 pub mod buffer_init;
+pub mod unapprove;
 
+pub use approve::*;
 pub use buffer_append_ix::*;
 pub use buffer_close::*;
 pub use buffer_execute_bundle::*;
